#ifndef __MERLIN_IBD_H__
#define __MERLIN_IBD_H__

#include "MathMatrix.h"
#include "Pedigree.h"
#include "Mantra.h"
#include "Tree.h"
#include "MerlinCore.h"

#include <stdio.h>

class MerlinIBD
   {
   public:
      MerlinIBD(Mantra & m) : mantra(m) {}

      double Calculate(Tree & tree, const char * label);
      double Score(Tree & tree, int node, int bit, int start = 0);
      void   Output(const char * label, double scale);

      void   OpenFile();
      void   CloseFile();

      void   SelectFamily(Pedigree * p, Family * f);

   private:
      Pedigree * ped;
      Family   * family;
      Mantra   & mantra;

      // Specialized routines for trees with symmetries
      double WeightedScore(Tree & tree, int node, int bit, int start,
                           double weight = 2.0);
      double PairWiseIBD1(int a, int b, int c, int d);
      double PairWiseIBD2(int a, int b, int c, int d);

      // Output files
      FILE     * ibdfile;

      // Temporary storage
      Matrix   IBD1, IBD2;

      // Uninformative meiosis in trees with symmetries
      IntArray uninformative;
   };

#endif

 
